"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const workspace_1 = require("@teambit/workspace");
const bundler_1 = require("@teambit/bundler");
const ui_1 = require("@teambit/ui");
const webpack_1 = require("@teambit/webpack");
const react_1 = __importDefault(require("react"));
const open_1 = __importDefault(require("open"));
const ink_1 = require("ink");
const humanize_duration_1 = __importDefault(require("humanize-duration"));
const moment_1 = __importDefault(require("moment"));
const bit_start_cmd_output_templates_1 = require("./bit-start-cmd-output-templates");
class StartCmd {
    constructor(
    /**
     * access to the extension instance.
     */
    ui, logger, pubsub) {
        this.ui = ui;
        this.logger = logger;
        this.pubsub = pubsub;
        this.items = [];
        this.onComponentChanges = [];
        this.nobrowser = false;
        this.devServerCounter = 0;
        this.targetHost = 'localhost';
        this.targetPort = 3000;
        this.name = 'start [type] [pattern]';
        this.description = 'Start a dev environment for a workspace or a specific component';
        this.alias = 'c';
        this.group = 'component';
        this.shortDescription = '';
        this.options = [
            ['d', 'dev', 'start UI server in dev mode.'],
            ['p', 'port', 'port of the UI server.'],
            ['r', 'rebuild', 'rebuild the UI'],
            ['v', 'verbose', 'showing verbose output for inspection and prints stack trace'],
            ['', 'suppress-browser-launch', 'do not automatically open browser when ready'],
        ];
        this.eventsListeners = (event) => {
            switch (event.type) {
                case bundler_1.ComponentsServerStartedEvent.TYPE:
                    // Do not remove thetimeout or the component might be removed by theClearConsole(!)
                    setTimeout(() => this.onComponentsServerStarted(event), 300);
                    break;
                // case 'webpack-compilation-done':
                case webpack_1.WebpackCompilationDoneEvent.TYPE:
                    // Do not remove thetimeout or the component might be removed by theClearConsole(!)
                    setTimeout(() => this.onWebpackCompilationDone(event), 0);
                    break;
                case ui_1.UiServerStartedEvent.TYPE:
                    this.onUiServerStarted(event);
                    break;
                case workspace_1.OnComponentChangeEvent.TYPE:
                    this.onComponentChange(event);
                    break;
                case workspace_1.OnComponentAddEvent.TYPE:
                    this.OnComponentAdd(event);
                    break;
                case workspace_1.OnComponentRemovedEvent.TYPE:
                    this.OnComponentRemoved(event);
                    break;
                default:
            }
        };
        this.OnComponentRemoved = (event) => {
            this.onComponentChange(event);
        };
        this.OnComponentAdd = (event) => {
            this.onComponentChange(event);
        };
        this.onComponentChange = (event) => {
            const { hook, idStr } = event.data;
            this.onComponentChanges.push({
                hook,
                idStr,
                timestamp: moment_1.default().format('HH:mm:ss'),
            });
            ink_1.render(react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement(bit_start_cmd_output_templates_1.ComponentPreviewServerStarted, { items: this.items }),
                react_1.default.createElement(bit_start_cmd_output_templates_1.StandaloneNewLine, null),
                react_1.default.createElement(bit_start_cmd_output_templates_1.UIServersAreReady, { host: this.targetHost, port: this.targetPort, timestamp: this.getDuration(), workspace: workspace_1.WorkspaceAspect }),
                react_1.default.createElement(bit_start_cmd_output_templates_1.StandaloneNewLine, null),
                react_1.default.createElement(bit_start_cmd_output_templates_1.OnComponentChange, { events: this.onComponentChanges })));
        };
        this.onUiServerStarted = (event) => __awaiter(this, void 0, void 0, function* () {
            this.targetHost = event.data.targetHost;
            this.targetPort = event.data.targetPort;
            const devServers = yield event.data.uiRoot.devServers;
            if (!devServers.length) {
                // no dev servers to wait for.
                this.openBrowserOn0();
            }
        });
        // eslint-disable-next-line no-unused-vars, @typescript-eslint/no-unused-vars
        this.onWebpackCompilationDone = (_event) => {
            this.devServerCounter -= 1;
            this.openBrowserOn0();
        };
    }
    registerToEvents() {
        this.pubsub.sub('teambit.bit/ui', this.eventsListeners);
        this.pubsub.sub('teambit.bit/webpack', this.eventsListeners);
        this.pubsub.sub('teambit.bit/bundler', this.eventsListeners);
        this.pubsub.sub('teambit.bit/workspace', this.eventsListeners);
    }
    onComponentsServerStarted(event) {
        this.devServerCounter += 1;
        this.items.push({
            envName: event.data.executionContext.envRuntime.id,
            host: event.data.hostname,
            port: event.data.port,
            timestamp: this.getDuration(),
        });
        ink_1.render(react_1.default.createElement(react_1.default.Fragment, null,
            react_1.default.createElement(bit_start_cmd_output_templates_1.ComponentPreviewServerStarted, { items: this.items }),
            react_1.default.createElement(bit_start_cmd_output_templates_1.StandaloneNewLine, null),
            react_1.default.createElement(bit_start_cmd_output_templates_1.StartingMainUiServer, { workspace: workspace_1.WorkspaceAspect })));
    }
    openBrowserOn0() {
        if (this.devServerCounter === 0) {
            ink_1.render(react_1.default.createElement(react_1.default.Fragment, null,
                react_1.default.createElement(bit_start_cmd_output_templates_1.ComponentPreviewServerStarted, { items: this.items }),
                react_1.default.createElement(bit_start_cmd_output_templates_1.StandaloneNewLine, null),
                react_1.default.createElement(bit_start_cmd_output_templates_1.UIServersAreReady, { host: this.targetHost, port: this.targetPort, timestamp: this.getDuration(), workspace: workspace_1.WorkspaceAspect })));
            if (!this.nobrowser)
                setTimeout(() => open_1.default(`http://${this.targetHost}:${this.targetPort}/`), 500);
        }
    }
    getDuration() {
        const duration = Date.now() - this.startingtimestamp;
        return humanize_duration_1.default(duration);
    }
    render([uiRootName, userPattern], { dev, port, rebuild, verbose, suppressBrowserLaunch, }) {
        return __awaiter(this, void 0, void 0, function* () {
            this.startingtimestamp = Date.now();
            this.nobrowser = !!suppressBrowserLaunch;
            // TODO[uri]: move outside when refactor to react app.
            this.registerToEvents();
            const pattern = userPattern && userPattern.toString();
            this.logger.off();
            yield this.ui.createRuntime({
                uiRootName,
                pattern,
                dev,
                port: port ? parseInt(port) : undefined,
                rebuild,
            });
            return (react_1.default.createElement(react_1.default.Fragment, null,
                verbose ? react_1.default.createElement(bit_start_cmd_output_templates_1.StandaloneNewLine, null) : react_1.default.createElement(bit_start_cmd_output_templates_1.ClearConsole, null),
                react_1.default.createElement(bit_start_cmd_output_templates_1.Starting, { workspace: workspace_1.WorkspaceAspect })));
        });
    }
}
exports.StartCmd = StartCmd;
//# sourceMappingURL=start.cmd.js.map